#! /usr/bin/env python
import sys
import json
from harpoon.commands.base import Command
from harpoon.lib.utils import typeguess
from OTXv2 import OTXv2, IndicatorTypes

OTX_TYPES = {
    "domain": IndicatorTypes.DOMAIN,
    "IPv4": IndicatorTypes.IPv4,
    "IPv6": IndicatorTypes.IPv6,
    "hostname": IndicatorTypes.HOSTNAME,
    "url": IndicatorTypes.URL,
    "md5": IndicatorTypes.FILE_HASH_MD5,
    "sha1": IndicatorTypes.FILE_HASH_SHA1,
    "sha256": IndicatorTypes.FILE_HASH_SHA256,
    "pehash": IndicatorTypes.FILE_HASH_PEHASH,
    "imphash": IndicatorTypes.FILE_HASH_IMPHASH,
    "cidr": IndicatorTypes.CIDR,
    "file_path": IndicatorTypes.FILE_PATH,
    "hostname": IndicatorTypes.HOSTNAME,
    "mutex": IndicatorTypes.MUTEX,
    "cve": IndicatorTypes.CVE
}


class CommandOtx(Command):
    """
    # AlienVault Open Threat Exchange

    **Requests information from AlienVault Open Threat Exchange https://www.alienvault.com/open-threat-exchange**

    * Search for an indicator: `harpoon otx -s 127.0.0.1`
    * Search for a list of indicators from a file: `harpoon otx -f FILE`
    * Get information on a pulse: `harpoon otx -p PULSE_ID`

    You can get raw JSON output with `-j`, and gives the IOC type with `-t`
    """
    name = "otx"
    description = "Requests information from AlienVault OTX"
    config = { 'AlienVaultOtx': ['key']}

    def add_arguments(self, parser):
        parser.add_argument('--pulse', '-p',  help='Event infos')
        parser.add_argument('--search', '-s',  help='Search for indicators')
        parser.add_argument('--file', '-f',  help='Check from a list of indicators in a file')
        parser.add_argument('--raw', '-r', help='Print raw information', action='store_true')
        parser.add_argument('--json', '-j', help='Print json information', action='store_true')
        parser.add_argument('--type', '-t', help='Type for search', default="guess", choices=["guess", "domain", "IPv4", "IPv6", "url", "md5", "sha1", "sha256", "pehash", "imphash", "cidr", "file_path", "hostname", "mutex", "cve"])
        self.parser = parser

    def run(self, conf, args, plugins):
        otx = OTXv2(conf["AlienVaultOtx"]["key"])
        if args.pulse:
            try:
                indicators = otx.get_pulse_indicators(args.pulse)
            except Exception:
                print("Pulse not found")
            else:
                if args.json:
                    print(json.dumps(indicators, sort_keys=True, indent=4))
                elif args.raw:
                    for i in indicators:
                        print(i["indicator"])
                else:
                    for i in indicators:
                        if len(i["type"]) < 5:
                            t = "\t\t\t"
                        else:
                            t = "\t\t"

                        print("[+] %s %s%s%s" % (
                                i["created"],
                                i["type"],
                                t,
                                i["indicator"]
                            )
                        )
        elif args.search:
            if args.type == "guess":
                t = typeguess(args.search)
            else:
                t = args.type

            res = otx.get_indicator_details_full(
                OTX_TYPES[t],
                args.search
            )
            if args.json:
                print(json.dumps(res, sort_keys=True, indent=4))
            else:
                if 'analysis' in res:
                    if res['analysis']['analysis']:
                        analysis = res["analysis"]["analysis"]
                        print("File analysed the %s" % analysis["datetime_int"])
                        if "info" in analysis:
                            print("Infos:")
                            for r in analysis["info"]["results"]:
                                print("\t%s : %s" % (r, analysis["info"]["results"][r]))
                        if "exiftool" in analysis["plugins"]:
                            print("Exiftool:")
                            for r in analysis["plugins"]["exiftool"]["results"]:
                                print("\t%s : %s" % (r, analysis["plugins"]["exiftool"]["results"][r]))
                        if "yarad" in analysis["plugins"]:
                            print("Yara detection:")
                            for r in analysis["plugins"]["yarad"]["results"]['detection']:
                                print("\t%s : %s" % (r["category"], r["rule_name"]))

                        print("")
                    else:
                        print("No analysis on this file")
                else:
                    print("No analysis on this file")
                if len(res["general"]["pulse_info"]["pulses"]) > 0:
                    print("Listed in %s pulses" % len(res["general"]["pulse_info"]["pulses"]))
                    for p in res["general"]["pulse_info"]["pulses"]:
                        print("\t-%s" % p["name"])
                        print("\t\t%s" % p["description"].replace("\n", " "))
                        print("\t\tCreated: %s" % p["created"])
                        print("\t\tReferences: %s" % ", ".join(p["references"]))
                        print("\t\tid: %s" % p["id"])
                else:
                    print("Not listed in any pulse")
                if 'passive_dns' in res:
                    if len(res['passive_dns']['passive_dns']) > 0:
                        print("Passive DNS:")
                        for p in res['passive_dns']['passive_dns']:
                            print("\t%s%s on IP %s [%s -> %s]" % (
                                    p["hostname"],
                                    p["indicator_link"],
                                    p["address"],
                                    p["first"],
                                    p["last"]
                                )
                            )
                        print("")
                if 'url_list' in res:
                    if len(res['url_list']['url_list']) > 0:
                        print("URL list:")
                        for u in res['url_list']['url_list']:
                            if "result" in u:
                                if "urlworker" in u["result"]:
                                    if "ip" in u["result"]["urlworker"] :
                                        print("\t[%s] %s on IP %s" % (
                                                u["date"],
                                                u["url"],
                                                u["result"]["urlworker"]["ip"]
                                            )
                                        )
                                    else:
                                        print("\t[%s] %s" % (
                                                u["date"],
                                                u["url"]
                                            )
                                        )
                                else:
                                    print("\t[%s] %s" % (u["date"], u["url"]))
                            else:
                                print("\t[%s] %s" % (u["date"], u["url"]))
                        print("")
        elif args.file:
            with open(args.file, "r") as f:
                data = f.read().split("\n")
            for d in data:
                if d.strip() != "":
                    if args.type == "guess":
                        t = typeguess(d.strip())
                    else:
                        t = args.type

                    print("========== %s" % d.strip())
                    res = otx.get_indicator_details_full(
                        OTX_TYPES[t],
                        d.strip()
                    )
                    if len(res["general"]["pulse_info"]["pulses"]) > 0:
                        print("Listed in %s pulses" % len(res["general"]["pulse_info"]["pulses"]))
                        for p in res["general"]["pulse_info"]["pulses"]:
                            print("\t-%s" % p["name"])
                            print("\t\t%s" % p["description"].replace("\n", " "))
                            print("\t\tCreated: %s" % p["created"])
                            print("\t\tReferences: %s" % ", ".join(p["references"]))
                            print("\t\tid: %s" % p["id"])
                    else:
                        print("Not listed in any pulse")
        else:
            self.parser.print_help()



#! /usr/bin/env python
import sys
import json
import shodan
from harpoon.commands.base import Command
from dateutil.parser import parse


class CommandShodan(Command):
    """
    # Shodan

    **Queries information from shodan.io API***

    * Get information on an IP : `harpoon shodan ip IP`
    * Get summary (only ports 22, 80 and 443) of historical data on an ip : `harpoon shodan ip -H -s IP`
    * Get raw json of historical data : `harpoon shodan ip -H -v IP`
    * Search in the database: `harpoon shodan search SEARCH`
    """
    name = "shodan"
    description = "Requests Shodan API"
    config = {'Shodan': ['key']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('ip', help='Get information on an IP address')
        parser_a.add_argument('IP', help='IP to be searched')
        parser_a.add_argument('--history', '-H', action='store_true',
                help='Also display historical information')
        parser_a.add_argument('-v', '--verbose', action='store_true',
                help="Verbose mode (display raw json)")
        parser_a.add_argument('-s', '--summary', action='store_true',
                help="Only display information for ports 22, 80 and 443")
        parser_a.set_defaults(subcommand='ip')
        parser_b = subparsers.add_parser('search', help='Search in shodan')
        parser_b.add_argument('QUERY', help='Query')
        parser_b.set_defaults(subcommand='search')
        parser_c = subparsers.add_parser('ssh', help='Write ssh history from Shodan historical data')
        parser_c.add_argument('IP', help='IP address')
        parser_c.set_defaults(subcommand='ssh')
        self.parser = parser

    def run(self, conf, args, plugins):
        if 'subcommand' in args:
            if 'Shodan' not in conf and 'key' not in conf['Shodan']:
                print('Bad configuration for Shodan, quitting...')
                sys.exit(1)
            api = shodan.Shodan(conf['Shodan']['key'])
            if args.subcommand == 'ip':
                try:
                    res = api.host(args.IP, history=args.history)
                except shodan.exception.APIError:
                    print("IP not found in Shodan")
                else:
                    if args.verbose:
                        print(json.dumps(res, sort_keys=True, indent=4))
                    else:
                        if args.summary:
                            for d in res['data']:
                                if d['port'] == 22:
                                    print("%s - port 22 ssh - %s" % (
                                            d['timestamp'][:19],
                                            d['data'].split("\n")[0]
                                        )
                                    )
                                elif d['port'] == 80:
                                    print("%s - port 80 http - Server \"%s\"" % (
                                            d['timestamp'][:19],
                                            d['http']['server']
                                        )
                                    )
                                elif d['port'] == 443:
                                    if 'cert' in d['ssl']:
                                        print("%s - port 443 https - Cert \"%s\" \"%s\" %s - Server \"%s\"" % (
                                                d['timestamp'][:19],
                                                d['ssl']['cert']['subject']['CN'],
                                                d['ssl']['cert']['issuer']['CN'],
                                                d['ssl']['cert']['fingerprint']['sha1'],
                                                d['http']['server']
                                            )
                                        )
                                    else:
                                        print("%s - port 443 https - Cert Unknown- Server \"%s\"" % (
                                                d['timestamp'][:19],
                                                d['http']['server']
                                            )
                                        )
                        else:
                            for d in res['data']:
                                print(d['timestamp'])
                                print(d['_shodan']['module'])
                                print("%s/%i" % (d['transport'], d['port']))
                                print(d['data'])
                                if 'html' in d:
                                    print(d['html'][:2000])
                                if 'http' in d:
                                    print(json.dumps(d['http'])[:3000])
                                print('')

            elif args.subcommand == 'search':
                res = api.search(args.QUERY)
                print('%i results' % res['total'])
                for r in res['matches']:
                    print('[+] %s (%s): port %s/%i -> %s\n' % (
                            r['ip_str'],
                            r['org'],
                            r['transport'],
                            r['port'],
                            r['data'][:1000]
                        )
                    )
            elif args.subcommand == 'ssh':
                data = {}
                try:
                    res = api.host(args.IP, history=True)
                except shodan.exception.APIError:
                    print("IP not found in Shodan")
                else:
                    for event in res['data']:
                        if event['_shodan']['module'] == 'ssh':
                            if 'ssh' in event:
                                fingerprint = event['ssh']['fingerprint']
                                date = parse(event['timestamp'])
                                if fingerprint not in data:
                                    data[fingerprint] = {
                                        'first': date,
                                        'last': date,
                                        'fingerprint': fingerprint
                                    }
                                else:
                                    if data[fingerprint]['first'] > date:
                                        data[fingerprint]['first'] = date
                                    if data[fingerprint]['last'] < date:
                                        data[fingerprint]['last'] = date

                    for val in sorted(data.values(), key=lambda x:x['first']):
                        print('%s - %s -> %s' % (
                                val['fingerprint'],
                                val['first'].strftime('%Y-%m-%d'),
                                val['last'].strftime('%Y-%m-%d')
                            )
                        )
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import sys
import json
import datetime
from harpoon.commands.base import Command
from harpoon.lib.utils import bracket, unbracket
from passivetotal.libs.whois import WhoisRequest
from passivetotal.libs.dns import DnsRequest
from passivetotal.libs.enrichment import EnrichmentRequest


class CommandPassiveTotal(Command):
    """
    # Passive Total

    * Get a domain whois: `harpoon pt whois -d example.org`
    * Search for a domain registered by an email address: `harpoon pt whois -e admin@example.org`
    * Get whois information for a domain list in a file: `harpoon pt whois -f FILE`
    * Query DNS information for a domain: `harpoon pt dns example.org`
    * Check malware related to a domain: `harpoon pt malware -d example.org`
    * Check report related to a domain: `harpoon pt osint -d example.org`
    """
    name = "pt"
    description = "Requests Passive Total database"
    config = {'PassiveTotal': ['username', 'key']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('whois', help='Request whois info')
        parser_a.add_argument('--domain', '-d', help='DOMAIN to be queried')
        parser_a.add_argument('--file', '-f', help='File with list of domains')
        parser_a.add_argument('--email', '-e', help='Check for domain registered by this email')
        parser_a.set_defaults(subcommand='whois')
        parser_b = subparsers.add_parser('dns', help='Request dns info')
        parser_b.add_argument('DOMAIN',  help='DOMAIN to be queried')
        parser_b.set_defaults(subcommand='dns')
        parser_c = subparsers.add_parser('malware', help='Request malware info')
        parser_c.add_argument('--domain', '-d',  help='DOMAIN to be queried')
        parser_c.add_argument('--file', '-f',  help='Check malware info from a domain list in a file and return csv of results')
        parser_c.add_argument('--raw', '-r',  help='Show raw results (JSON)', action="store_true")
        parser_c.set_defaults(subcommand='malware')
        parser_d = subparsers.add_parser('osint', help='Request OSINT info')
        parser_d.add_argument('--domain', '-d',  help='DOMAIN to be queried')
        parser_d.add_argument('--file', '-f',  help='Check OSINT info from a domain list in a file and return csv of results')
        parser_d.add_argument('--raw', '-r',  help='Show raw results (JSON)', action="store_true")
        parser_d.set_defaults(subcommand='osint')
        self.parser = parser


    def run(self, conf, args, plugins):
        if 'subcommand' in args:
            if args.subcommand == 'whois':
                client = WhoisRequest(conf['PassiveTotal']['username'], conf['PassiveTotal']['key'])
                if args.domain:
                    raw_results = client.search_whois_by_field(
                        query=unbracket(args.domain.strip()),
                        field="domain"
                    )
                    print(json.dumps(raw_results,  sort_keys=True, indent=4, separators=(',', ': ')))
                elif args.file:
                    with open(args.file, 'r') as infile:
                        data = infile.read().split()
                    print("Domain|Date|Registrar|name|email|Phone|organization|Street|City|Postal Code|State|Country")
                    for d in data:
                        do = unbracket(d.strip())
                        # FIXME: bulk request here
                        raw_results = client.search_whois_by_field(
                            query=do,
                            field="domain"
                        )
                        if "results" not in raw_results:
                            print("%s|||||||||||" %  bracket(do) )
                        else:
                            if len(raw_results["results"]) == 0:
                                print("%s|||||||||||" %  bracket(do) )
                            else:
                                r = raw_results["results"][0]
                                if "registered" in r:
                                    dd = datetime.datetime.strptime(r["registered"], "%Y-%m-%dT%H:%M:%S.%f%z")
                                    ddo = dd.strftime("%m/%d/%Y %H:%M:%S")
                                else:
                                    ddo = ""

                                print("%s|%s|%s|%s|%s|%s|%s|%s|%s|%s|%s|%s" % (
                                        bracket(do),
                                        ddo,
                                        r["registrar"] if "registrar" in r else "",
                                        r["registrant"]["name"] if "name" in r["registrant"] else "",
                                        r["registrant"]["email"] if "email" in r["registrant"] else "",
                                        r["registrant"]["telephone"] if "telephone" in r["registrant"] else "",
                                        r["registrant"]["organization"] if "organization" in r["registrant"] else "",
                                        r["registrant"]["street"] if "street" in r["registrant"] else "",
                                        r["registrant"]["city"] if "city" in r["registrant"] else "",
                                        r["registrant"]["postalCode"] if "postalCode" in r["registrant"] else "",
                                        r["registrant"]["state"] if "state" in r["registrant"] else "",
                                        r["registrant"]["country"] if "country" in r["registrant"] else ""
                                    )
                                )


                elif args.email:
                    raw_results = client.search_whois_by_field(
                        query=args.email.strip(),
                        field="email"
                    )
                    print(json.dumps(raw_results,  sort_keys=True, indent=4, separators=(',', ': ')))
                else:
                    self.parser.print_help()
            elif args.subcommand == "dns":
                client = DnsRequest(conf['PassiveTotal']['username'], conf['PassiveTotal']['key'])
                raw_results = client.get_passive_dns(
                    query=unbracket(args.DOMAIN),
                )
                print(json.dumps(raw_results,  sort_keys=True, indent=4, separators=(',', ': ')))
            elif args.subcommand == "malware":
                client = EnrichmentRequest(conf["PassiveTotal"]["username"], conf["PassiveTotal"]['key'])
                if args.domain:
                    raw_results = client.get_malware(query=args.domain)
                    print(json.dumps(raw_results,  sort_keys=True, indent=4, separators=(',', ': ')))
                elif args.file:
                    with open(args.file, 'r') as infile:
                        data = infile.read().split()
                    domain_list = list(set([a.strip() for a in data]))
                    if len(domain_list) < 51:
                        raw_results = client.get_bulk_malware(query=domain_list)
                        if "results" not in raw_results or not raw_results["success"]:
                            print("Request failed")
                            print(json.dumps(raw_results,  sort_keys=True, indent=4, separators=(',', ': ')))
                            sys.exit(1)
                        else:
                            results = raw_results["results"]
                    else:
                        results = {}
                        bulk_size=50
                        i = 0
                        while i*bulk_size < len(domain_list):
                            raw_results = client.get_bulk_malware(query=domain_list[i*bulk_size:(i+1)*bulk_size])
                            if "results" not in raw_results or not raw_results["success"]:
                                print("Request failed")
                                print(json.dumps(raw_results,  sort_keys=True, indent=4, separators=(',', ': ')))
                                sys.exit(1)
                            else:
                                results.update(raw_results["results"])
                            i += 1
                    if args.raw:
                        print(json.dumps(results, sort_keys=True, indent=4, separators=(',', ': ')))
                    else:
                        print("Domain|Date|Sample|Source|Source URL")
                        for domain in results:
                            if "results" in results[domain]:
                                for sample in results[domain]["results"]:
                                    print("%s|%s|%s|%s|%s" % (
                                                domain,
                                                sample["collectionDate"],
                                                sample["sample"],
                                                sample["source"],
                                                sample["sourceUrl"]
                                            )
                                        )

                else:
                    self.parser.print_help()

            elif args.subcommand == "osint":
                # FIXME: add research of projects
                client = EnrichmentRequest(conf["PassiveTotal"]["username"], conf["PassiveTotal"]['key'])
                if args.domain:
                    raw_results = client.get_osint(query=args.domain)
                    print(json.dumps(raw_results,  sort_keys=True, indent=4, separators=(',', ': ')))
                elif args.file:
                    with open(args.file, 'r') as infile:
                        data = infile.read().split()
                    domain_list = list(set([a.strip() for a in data]))
                    if len(domain_list) < 51:
                        raw_results = client.get_bulk_osint(query=domain_list)
                        if "results" not in raw_results or not raw_results["success"]:
                            print("Request failed")
                            print(json.dumps(raw_results,  sort_keys=True, indent=4, separators=(',', ': ')))
                            sys.exit(1)
                        else:
                            results = raw_results["results"]
                    else:
                        results = {}
                        bulk_size=50
                        i = 0
                        while i*bulk_size < len(domain_list):
                            raw_results = client.get_bulk_osint(query=domain_list[i*bulk_size:(i+1)*bulk_size])
                            if "results" not in raw_results or not raw_results["success"]:
                                print("Request failed")
                                print(json.dumps(raw_results,  sort_keys=True, indent=4, separators=(',', ': ')))
                                sys.exit(1)
                            else:
                                results.update(raw_results["results"])
                            i += 1
                    if args.raw:
                        print(json.dumps(results, sort_keys=True, indent=4, separators=(',', ': ')))
                    else:
                        print("Domain|Source|URL|Tags")
                        for domain in results:
                            if "results" in results[domain]:
                                for report in results[domain]["results"]:
                                    print("%s|%s|%s|%s" % (
                                            domain,
                                            report["source"],
                                            report["source_url"],
                                            " / ".join(report["tags"])
                                        )
                                    )
                else:
                    self.parser.print_help()

            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
from harpoon.commands.base import Command
from harpoon.commands.censyscmd import CommandCensys
from harpoon.lib.utils import unbracket
from passivetotal.libs.enrichment import EnrichmentRequest
from virus_total_apis import PublicApi, PrivateApi
from censys.base import CensysRateLimitExceededException


class CommandSubdomains(Command):
    """
    # Subdomains plugin

    **Research subdomains for a domain**

    * Search for subdomains : `harpoon subdomains DOMAIN`

    """
    name = "subdomains"
    description = "Research subdomains of a domain"

    def add_arguments(self, parser):
        parser.add_argument('DOMAIN', help='Domain')
        parser.add_argument('--verbose', '-v', action='store_true', help='Verbose mode')
        parser.add_argument(
            '--source', '-s',
            choices=['all', 'censys', 'pt', 'vt'],
            default='all',
            help='Source of the research'
        )
        self.parser = parser

    def censys_certs(self, domain, conf, verbose):
        censys_cmd = CommandCensys()
        print('## Searching through Censys certificates')
        subdomains = censys_cmd.get_subdomains(
            conf,
            domain,
            verbose,
            only_sub=True
        )
        for d in subdomains:
            print(d)

    def pt(self, domain, conf, verbose):
        client = EnrichmentRequest(
            conf["PassiveTotal"]["username"],
            conf["PassiveTotal"]['key']
        )
        print('## Searching subdomains in Passive Total')
        res = client.get_subdomains(query=domain)
        for d in res['subdomains']:
            print('%s.%s' % (d, domain))

    def vt(self, domain, conf, verbose):
        print('## Searching subdomains in Virus Total')
        if conf["VirusTotal"]["type"] == "public":
            vt = PublicApi(conf["VirusTotal"]["key"])
        else:
            vt = PrivateApi(conf["VirusTotal"]["key"])
        res = vt.get_domain_report(domain)
        try:
            for d in res['results']['subdomains']:
                print(d)
        except KeyError:
            pass

    def run(self, conf, args, plugins):
        if args.source == 'all':
            # Search subdomains through a search in Censys certificates
            if plugins['censys'].test_config(conf):
                try:
                    self.censys_certs(unbracket(args.DOMAIN), conf, args.verbose)
                except CensysRateLimitExceededException:
                    print('Quota exceeded!')
            # Get subdomains through passive total
            if plugins['pt'].test_config(conf):
                self.pt(unbracket(args.DOMAIN), conf, args.verbose)
            if plugins['vt'].test_config(conf):
                self.vt(unbracket(args.DOMAIN), conf, args.verbose)

        elif args.source == 'censys':
            if plugins['censys'].test_config(conf):
                try:
                    self.censys_certs(unbracket(args.DOMAIN), conf, args.verbose)
                except CensysRateLimitExceededException:
                    print('Quota exceeded!')
            else:
                print('Please configure your Censys credentials')
        elif args.source == 'pt':
            if plugins['pt'].test_config(conf):
                self.pt(unbracket(args.DOMAIN), conf, args.verbose)
            else:
                print('Please configure your Passive Total credentials')
        elif args.source == 'vt':
            if plugins['vt'].test_config(conf):
                self.vt(unbracket(args.DOMAIN), conf, args.verbose)
            else:
                print('Please configure your VirusTotal credentials')



#! /usr/bin/env python
import sys
import json
import requests
from harpoon.commands.base import Command
import phonenumbers

class CommandNumVerify(Command):
    """
    # Numverify.com plugin

    **Query NumVerify API**

    * Check a phone number
    """
    name = "numverify"
    description = "Query phone number information from NumVerify"
    config = { 'NumVerify': ['key']}

    def query(self, phone, cc, key):
        params = {
                "access_key": key,
                "number": phone,
                "country_code": cc,
                "format": 1
        }
        r = requests.get("http://apilayer.net/api/validate", params=params)
        return r.json()

    def add_arguments(self, parser):
        parser.add_argument('PHONE',  help='Phone number')
        parser.add_argument('COUNTRY',  help='Country code (two letters)')
        self.parser = parser

    def run(self, conf, args, plugins):
        x = phonenumbers.parse(args.PHONE, args.COUNTRY)
        print("Requesting phone +{} - {} ({})".format(
            x.country_code,
            x.national_number,
            args.COUNTRY
        ))
        res = self.query(x.national_number, args.COUNTRY, conf['NumVerify']['key'])
        print(json.dumps(res, sort_keys=False, indent=4))



#! /usr/bin/env python
import sys
from harpoon.commands.base import Command
from harpoon.lib.pgp import Pgp


class CommandPgp(Command):
    """
    # PGP

    **Searches for PGP keys in MIT key servers**

    Example:
    ```
    harpoon pgp search lemonde.fr
    [+] 0x07A0514E0F568618	2017-12-19	Anne Michel anne.michel@lemonde.fr
    [+] 0xAE95106F1B5A0D7E	2017-11-28	perso maxime.loliee@gmail.com
					loliee maxime.loliee@gmail.com
					pro maxime@siliadev.com
					lemonde loliee@lemonde.fr
    [SNIP]
    ```

    Option `-o` just print the list of emails (no information on keys)
    """
    name = "pgp"
    description = "Search for information in PGP key servers"
    config = {}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommands')
        parser_a = subparsers.add_parser('search', help='Search in PGP server')
        parser_a.add_argument('SEARCH', help='Query')
        parser_a.add_argument('--only-emails', '-o', action='store_true',
                            help='Print only email addresses')
        parser_a.set_defaults(subcommand='search')
        self.parser = parser

    def run(self, conf, args, plugins):
        if hasattr(args, 'subcommand'):
            if args.subcommand == "search":
                res = Pgp.search(args.SEARCH)
                if len(res):
                    if args.only_emails:
                        emails = set()
                        for r in res:
                            for e in r['emails']:
                                if args.SEARCH.lower() in e[1].lower():
                                    emails.add(e[1])
                        for e in emails:
                            print(e)
                    else:
                        for r in res:
                            print("[+] %s\t%s\t%s %s" % (
                                    r['id'],
                                    r['date'].strftime("%Y-%m-%d"),
                                    r['emails'][0][0],
                                    r['emails'][0][1]
                                )
                            )
                            if len(r['emails']) > 1:
                                for e in r['emails'][1:]:
                                    print("\t\t\t\t\t%s %s" % (e[0], e[1]))
                else:
                    print("No results (could be too many results)")
                    print("Double check at http://pgp.mit.edu/pks/lookup?search=%s" % args.SEARCH)
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import sys
from harpoon.commands.base import Command
from pysafebrowsing import SafeBrowsing

class CommandSafeBrowsing(Command):
    """
    # Google Safe Browsing

    **Check online Safe Browsing information**

    * Query an url: `harpoon safebrowsing url URL`
    * Query a list of domains or url from a file with CSV output: `harpoon safebrowsing file FILE -f csv`
    """
    name = "safebrowsing"
    description = "Check if the given domain is in Google safe Browsing list"
    config = {'SafeBrowsing': ['key']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='SubCommands')
        parser_b = subparsers.add_parser('url', help='Query an URL')
        parser_b.add_argument('URL', help='URL to be requested')
        parser_b.add_argument('--json', '-j', action='store_true', help='Show raw json')
        parser_b.set_defaults(subcommand='url')
        parser_c = subparsers.add_parser('file', help='Check domains or urls from a file')
        parser_c.add_argument('FILE', help='File path')
        parser_c.add_argument('--format', '-f', help='Output format',
            choices=["json", "csv", "txt"], default="txt")
        parser_c.set_defaults(subcommand='file')
        self.parser = parser

    def run(self, conf, args, plugins):
        sb = SafeBrowsing(conf['SafeBrowsing']['key'])
        if 'subcommand' in args:
            if args.subcommand == 'url':
                try:
                    if args.URL.startswith("http"):
                        res = sb.lookup_url(args.URL)
                    else:
                        res = sb.lookup_url("http://" + args.URL + "/")
                except SafeBrowsingInvalidApiKey:
                    print("Invalid API key!")
                    sys.exit(1)
                except SafeBrowsingWeirdError:
                    print("Weird Error!")
                    sys.exit(1)
                else:
                    if args.json:
                        print(json.dumps(res, sort_keys=True, indent=4))
                    else:
                        if res["malicious"]:
                            print("Malicious: Yes")
                            print("Platforms: %s" % ", ".join(res["platforms"]))
                            print("Threats: %s" % ", ".join(res["threats"]))
                        else:
                            print("Malicious: No")
            elif args.subcommand == 'file':
                with open(args.FILE, 'r') as f:
                    data = f.read()
                domains = [d.strip() for d in data.split()]
                res = sb.lookup_urls(domains)
                if args.format == "txt":
                    for domain in res:
                        if res[domain]["malicious"]:
                            print("%s\tMalicious" % domain)
                        else:
                            print("%s\tOk" % domain)
                elif args.format == "json":
                    print(json.dumps(res, sort_keys=True, indent=4))
                else:
                    print("Url|Malicious|Threat|Platform")
                    for domain in res:
                        if res[domain]["malicious"]:
                            print("%s|%s|%s|%s" % (
                                    domain,
                                    "Yes",
                                    ",".join(res[domain]["threats"]),
                                    ",".join(res[domain]["platforms"])
                                )
                            )
                        else:
                            print("%s|No||" % domain)
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import sys
import os
import json
from harpoon.commands.base import Command
from harpoon.lib.utils import unbracket
from harpoon.lib.cybercure import CyberCure, CyberCureError


class CommandCyberCure(Command):
    """
    # cybercure.ai plugin

    **Check if intelligence on an IP exists**

    Query cybercure API:
    ```
    harpoon cybercure ip 184.186.250.211
    {
    "exists": true,
    "indicator": "184.186.250.211",
    "status": 1,
    "visual": "http://www.cybercure.ai/intel/ip/184.186.250.211"
    }
    ```

    """
    name = "cybercure"
    description = "Search cybercure.ai intelligence database for specific indicators."
    #config = {'IPInfo': ['token']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('ip', help='Returns a response whether an indicator exists in cybercure.ai database, if it is exists it will provide also a link for visual presentation of the indicator history.')
        parser_a.add_argument('IP', help='IP address')
        parser_a.set_defaults(subcommand='ip')
        parser_b = subparsers.add_parser('file', help='Information on a list of IPs')
        parser_b.add_argument('FILE', help='Filename')
        parser_b.set_defaults(subcommand='file')
        self.parser = parser

    def run(self, conf, args, plugins):
        cybercure = CyberCure(token='reserved_for_future')
        if 'subcommand' in args:
            if args.subcommand == 'ip':
                try:
                    infos = cybercure.get_infos(unbracket(args.IP))
                except CyberCureError:
                    print("Invalid request")
                else:
                    print(json.dumps(infos,  sort_keys=True, indent=4, separators=(',', ': ')))
            elif args.subcommand == 'file':
                if os.path.isfile(args.FILE):
                    with open(args.FILE) as f:
                        data = f.read().split("\n")
                    print("IP;Exists;Details")
                    for d in data:
                        if d.strip() == '':
                            continue
                        ip = unbracket(d.strip())
                        try:
                            infos = cybercure.get_infos(ip)
                        except CyberCureError:
                            print("%s;;" % ip)
                        else:
                            print ("%s;%s;%s" % (
                                    ip,
                                    infos['exists'],
                                    infos['visual'] if 'visual' in infos else ''
                                )
                            )
                else:
                    print("This file does not exist")
                    sys.exit(1)
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import sys
from collections import Counter
from harpoon.commands.base import Command
from mispy import MispServer, MispEvent


class CommandMisp(Command):
    """
    # MISP

    **Queries a MISP (http://www.misp-project.org/) server**

    * List events (maybe a long list): `harpoon misp -l`
    * Get information on an event: `harpoon misp -e 342`
    * Search for an attribute: `harpoon misp -a example.org`
    * Search for a list of indicators in a file: `harpoon misp -s FILE`
    """
    name = "misp"
    description = "Get information from a MISP server through the API"
    config = { 'Misp': ['url', 'key']}

    def add_arguments(self, parser):
        parser.add_argument('--list', '-l', action='store_true', help='List events')
        parser.add_argument('--event', '-e',  help='Event infos', type=int)
        parser.add_argument('--search', '-s',  help='Search for indicators from a file')
        parser.add_argument('--attr', '-a',  help='Search for this attribute')
        parser.add_argument('--type', '-t',  help='Search for attributes of this type')
        parser.add_argument('--raw', '-r', help='Print raw information', action='store_true')
        parser.add_argument('--no-tls', '-n', help='Do not validate TLS certificate (bad bad bad)', action='store_true')
        self.parser = parser

    def run(self, conf, args, plugins):
        # FIXME: have this in conf
        if args.no_tls:
            server = MispServer(url=conf['Misp']['url'], apikey=conf['Misp']['key'], ssl_chain=False)
        else:
            server = MispServer(url=conf['Misp']['url'], apikey=conf['Misp']['key'])
        if args.list:
            # List events
            events = server.events.list(0)
            for event in sorted(events, key=lambda x:x.id):
                print("%i : %s" % (event.id, event.info))
        elif args.attr is not None:
            res = server.attributes.search(value=args.attr)
            if len(res) == 0:
                print("Search %s: no results" % args.attr)
            else:
                print("Search %s, result founds" % args.attr)
                for event in res:
                    print("[+] %i - %s" % (event.id, event.info))
                    for attr in event.attributes:
                        if args.type is not None:
                            if attr.type == args.type:
                                if args.attr.lower() in str(attr.value).lower() or \
                                        args.attr.lower() in str(attr.comment).lower():
                                    print("\t%s (%s / %s) %s" % (attr.value, attr.category, attr.type, attr.comment))
                        else:
                            if args.attr.lower() in str(attr.value).lower() or \
                                    args.attr.lower() in str(attr.comment).lower():
                                print("\t%s (%s / %s) %s" % (attr.value, attr.category, attr.type, attr.comment))


        elif args.event is not None:
            event = server.events.get(args.event)
            if args.attr is None and args.type is None:
                print("Event %i : %s" % (event.id, event.info))
                print("Tags : %s" % ", ".join(map(lambda x:str(x.name), event.tags)))
                print("%i Attributes including:" % len(event.attributes))
                attrs = Counter(map(lambda x:x.type, event.attributes))
                attrs_ids = Counter(map(lambda x:x.type, filter(lambda x:x.to_ids, event.attributes)))
                for type in attrs:
                    print("\t- %i %s (%i for detection)" % (attrs[type], type, attrs_ids[type]))
            else:
                if args.type is not None:
                    # Display all attributes from this type
                    for attr in event.attributes:
                        if attr.type == args.type:
                            if args.raw:
                                print("%s" % attr.value)
                            else:
                                print("%s\t%s\t%s\t%s\t%s" % (attr.category, attr.type, attr.value, attr.comment, attr.to_ids))
                elif args.attr is not None:
                    # search by attribute value
                    for attr in event.attributes:
                        if args.attr in str(attr.value):
                            print("%s\t%s\t%s\t%s\t%s" %
                                (
                                    attr.category,
                                    attr.type,
                                    attr.value,
                                    attr.comment,
                                    attr.to_ids
                                )
                            )
        elif args.search:
            with open(args.search, 'r') as infile:
                data = infile.read().split()
            for d in data:
                print("Searching for %s" % d.strip())
                res = server.attributes.search(value=d.strip())
                if len(res) == 0:
                    print("\tNo results")
                else:
                    for event in res:
                        print("\t[+] %i - %s" % (event.id, event.info))
                        for attr in event.attributes:
                            if d.strip().lower() in str(attr.value).lower() or \
                                d.strip().lower() in str(attr.comment).lower():
                                    print("\t\t%s (%s / %s) %s" % (attr.value, attr.category, attr.type, attr.comment))
        else:
            self.parser.print_help()



#! /usr/bin/env python
import sys
import json
import datetime
from harpoon.commands.base import Command
from harpoon.lib.utils import bracket, unbracket
from pysecuritytrails import SecurityTrails, SecurityTrailsError


class CommandSecurityTrails(Command):
    """
    # Security Trails

    Query https://securitytrails.com/

    * Get your quota : `harpoon securitytrails quota`
    * Get historical whois info on a domain : `harpoon securitytrails whois DOMAIN`
    * Get Passive DNS data on a domain : `harpoon securitytrails domain DOMAIN`
    * Get Passive DNS data on an IP : `harpoon securitytrails ip IP` (IPv4 only)
    * Get subdomains of a domain : `harpoon subdomains DOMAIN`

    """
    name = "securitytrails"
    description = "Requests SecurityTrails database"
    config = {'SecurityTrails': ['key']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('whois', help='Request whois info')
        parser_a.add_argument('DOMAIN', help='DOMAIN to be queried')
        parser_a.set_defaults(subcommand='whois')
        parser_b = subparsers.add_parser('quota', help='Check quota')
        parser_b.set_defaults(subcommand='quota')
        parser_c = subparsers.add_parser('domain', help='Passive DNS info')
        parser_c.add_argument('DOMAIN', help='DOMAIN to be queried')
        parser_c.set_defaults(subcommand='domain')
        parser_d = subparsers.add_parser('ip', help='Passive DNS info')
        parser_d.add_argument('IP', help='Search for passive DNS on an IP')
        parser_d.set_defaults(subcommand='ip')
        parser_e = subparsers.add_parser('subdomains', help='Get subdomains')
        parser_e.add_argument('DOMAIN', help='Domain to be queried')
        parser_e.set_defaults(subcommand='subdomains')
        self.parser = parser


    def run(self, conf, args, plugins):
        client = SecurityTrails(conf['SecurityTrails']['key'])
        if 'subcommand' in args:
            if args.subcommand == 'whois':
                res = client.domain_history_whois(args.DOMAIN)
                print(json.dumps(res['result']['items'], sort_keys=False, indent=4))
            elif args.subcommand == "quota":
                res = client.usage()
                print("Quota : {} / {}".format(
                        res['current_monthly_usage'],
                        res['allowed_monthly_usage']
                    )
                )
            elif args.subcommand == "domain":
                res = client.domain_history_dns(args.DOMAIN)
                print(json.dumps(res['records'], sort_keys=False, indent=4))
            elif args.subcommand == "ip":
                res = client.domain_search({"ipv4": args.IP}, include_ips=True)
                print(json.dumps(res['records'], sort_keys=False, indent=4))
            elif args.subcommand == "subdomains":
                res = client.domain_subdomains(args.DOMAIN)
                for d in res['subdomains']:
                    print(d + '.' + args.DOMAIN)
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



import requests
import html
from dateutil.parser import parse
from bs4 import BeautifulSoup
from urllib.parse import urlparse, parse_qs
from harpoon.lib.utils import same_url

class Google(object):
    @staticmethod
    def download_cache(url):
        """
        Download cache from a cache url
        """
        r = requests.get(url)
        if r.status_code == 200:
            mark1 = r.text.find("It is a snapshot of the page as it appeared on ")
            timestamptext = r.text[mark1+47:mark1+47+24]
            timestamp = parse(timestamptext)
            return {
                    "success": True,
                    "date": timestamp,
                    "data": html.unescape(r.text[r.text.find("<pre>")+5:r.text.find("</pre>")]),
                    'cacheurl': r.url

            }
        else:
            if r.status_code != 404:
                print("Weird, it should return 404...")
            return {"success": False}

    @staticmethod
    def search(query, num=10):
        payload = {
            "q": query,
            "num": num

        }
        r = requests.get(
            "https://www.google.com/search",
            params=payload
        )
        soup = BeautifulSoup(r.text, 'lxml')
        res = []
        divs = soup.find_all('div', class_='g')
        for d in divs:
            if d.h3:
                data = {
                    'name': d.h3.a.text
                }
                link = parse_qs(urlparse(d.h3.a['href']).query)
                if 'q' in link:
                    data['url'] = link['q'][0]
                else:
                    # Abnormal link (for instance Google books)
                    data['url'] = d.h3.a['href']
                text = d.find('span', class_='st')
                if text is not None:
                    data['text'] = text.text
                if d.ul:
                    for i in d.ul.children:
                        l = parse_qs(urlparse(i.a['href']).query)
                        if 'webcache.googleusercontent.com' in l['q'][0]:
                            data['cache'] = l['q'][0]
                res.append(data)
        return res

    @staticmethod
    def cache(url):
        payload = {
                "q": "cache:" + url,
                "num": 1,
                "strip":0,
                "vwsrc":1
        }
        r = requests.get(
            "https://webcache.googleusercontent.com/search",
            params=payload
        )
        if r.status_code == 200:
            # Copy code here to avoid doing another request
            mark1 = r.text.find("It is a snapshot of the page as it appeared on ")
            if mark1 > 0:
                timestamptext = r.text[mark1+47:mark1+47+24]
                timestamp = parse(timestamptext)
            if mark1 > 0:
                return {
                        "success": True,
                        "date": timestamp,
                        "data": html.unescape(r.text[r.text.find("<pre>")+5:r.text.find("</pre>")]),
                        'cacheurl': r.url,
                        'url': url
                }
            else:
                return {
                        "success": True,
                        "data": html.unescape(r.text[r.text.find("<pre>")+5:r.text.find("</pre>")]),
                        'cacheurl': r.url,
                        'url': url
                }
        else:
            # If not in cache directly, search on google for the cached url
            # Weirdly google does not find well url starting with a scheme
            if url.startswith('http://'):
                url = url[7:]
            elif url.startswith('https://'):
                url = url[8:]
            res = Google.search(url)
            for r in res:
                if same_url(url, r['url']) and 'cache' in r:
                    return Google.download_cache(r['cache'])
            return { 'success': False }



#! /usr/bin/env python
import sys
import json
from fullcontact import FullContact
from harpoon.commands.base import Command

class CommandFullContact(Command):
    """
    # FullContact

    **Request fullcontact.com API**

    * Search for a Twitter account: `harpoon fullcontact -t @tenacioustek`
    * Search for an email address: `harpoon fullcontact -e admin@example.org`
    * Search for a domain: `harpoon fullcontact -d github.com`
    """
    name = "fullcontact"
    description = "Requests Full Contact API (https://www.fullcontact.com/)"
    config = {'FullContact': ['key']}

    def add_arguments(self, parser):
        parser.add_argument('--twitter', '-t', help='Search person based on twitter')
        parser.add_argument('--email', '-e', help='Search person based on email')
        parser.add_argument('--md5', '-m', help='Search person based on md5 of email')
        parser.add_argument('--phone', '-p', help='Search person based on phone number')
        parser.add_argument('--domain', '-d', help='Search company based on domain')
        self.parser = parser

    def run(self, conf, args, plugins):
        fc = FullContact(conf['FullContact']['key'])
        if args.twitter:
            res = fc.person(twitter=args.twitter)
            print(json.dumps(res.json(), sort_keys=True, indent=4))
        elif args.email:
            res = fc.person(email=args.email)
            print(json.dumps(res.json(), sort_keys=True, indent=4))
        elif args.phone:
            res = fc.person(phone=args.phone)
            print(json.dumps(res.json(), sort_keys=True, indent=4))
        elif args.md5:
            res = fc.person(emailMD5=args.md5)
            print(json.dumps(res.json(), sort_keys=True, indent=4))
        elif args.domain:
            res = fc.person(domain=args.domain)
            print(json.dumps(res.json(), sort_keys=True, indent=4))
        else:
            self.parser.print_help()




class Command(object):
    config = None # Describes the configuration params
    update_needed = False

    @property
    def config_needed(self):
        return (self.config is not None)

    def update(self):
        pass

    def test_config(self, conf):
        """
        Test that the config params defined are in the conf file
        """
        if self.config is None:
            return True
        if len(self.config.keys()) == 0:
            return True
        else:
            for c in self.config:
                if c not in conf:
                    return False
                else:
                    for d in self.config[c]:
                        if d not in conf[c]:
                            return False
                        else:
                            if conf[c][d] == '':
                                return False
            return True

    def add_arguments(self, parser):
        pass



#! /usr/bin/env python
from harpoon.commands.base import Command
from harpoon.lib.utils import unbracket
from harpoon.lib.archiveis import ArchiveIs
from harpoon.lib.archiveorg import ArchiveOrg
from pypermacc import Permacc, PermaccError


class CommandSave(Command):
    """
    # Save plugin

    **Save an URL in cache platforms (archive.is, archive.org, perma.cc)**

    Commands:
    * Save an url: `harpoon save URL`
    """
    name = "save"
    description = "Save a webpage in cache platforms"
    config = {}

    def add_arguments(self, parser):
        parser.add_argument('URL', help='URL to save')
        self.parser = parser

    def run(self, conf, args, plugins):
        print("Saving in cache platforms:")
        # Archive.is
        try:
            ai_url = ArchiveIs.capture(unbracket(args.URL))
        except:
            print("Impossible to save in archive.is, weird")
        else:
            print("Archive.is: %s" % ai_url)

        # Web Archive
        try:
            ao_url = ArchiveOrg.capture(unbracket(args.URL))
        except:
            print("Impossible to save in web archive, weird.")
        else:
            print("Web Archive: %s" % ao_url)

        # Perma.cc
        if 'Permacc' in conf and 'key' in conf['Permacc']:
            pc = Permacc(conf['Permacc']['key'])
            try:
                saved = pc.archive_create(unbracket(args.URL))
            except PermaccError:
                print("Impossible to save in Permacc")
            else:
                print("Permacc: https://perma.cc/%s" % saved["guid"])



#! /usr/bin/env python
import sys
import os
import subprocess
from shutil import copyfile
from harpoon.commands.base import Command


class CommandConfig(Command):
    """
    # Configuration command

    **Help managing Harpoon configuration**

    * Create and update configuration file: `harpoon config`
    * Test plugins' configuration: `harpoon config -c`
    * Download or update databases: `harpoon config -u`

    Configuration file is stored in `~/.harpoon/config`
    """
    name = "config"
    description = "Configure Harpoon"

    def add_arguments(self, parser):
        parser.add_argument('--show', '-s', action='store_true',
            help='Show harpoon configuration')
        parser.add_argument('--check', '-c', action='store_true',
            help='Config harpoon configuration')
        parser.add_argument('--update', '-u', action='store_true',
            help='Update plugin data')

    def run(self, conf, args, plugins):
        configdir = os.path.join(os.path.expanduser('~'), '.config/harpoon')
        configpath = os.path.join(os.path.expanduser('~'), '.config/harpoon/config')
        if args.show:
            if not os.path.isfile(configpath):
                print('Config file does not exist, use harpoon config to create it')
                sys.exit(1)
            else:
                with open(configpath, 'r') as f:
                    print(f.read())
        elif args.check:
            print('Configuration check:')
            for p in plugins:
                if plugins[p].config_needed:
                    if plugins[p].test_config(conf):
                        if len(p) < 7:
                            print('-%s\t\t -> OK' % p)
                        else:
                            print('-%s\t -> OK' % p)
                    else:
                        if len(p) < 7:
                            print('-%s\t\t -> FAILED' % p)
                        else:
                            print('-%s\t -> FAILED' % p)
                else:
                    if len(p) < 7:
                        print('-%s\t\t -> OK' % p)
                    else:
                        print('-%s\t -> OK' % p)
        elif args.update:
            if not os.path.isdir(configdir):
                os.makedirs(configdir)
            print("Updating all plugins data:")
            for p in plugins:
                if plugins[p].update_needed:
                    print("Updating plugin %s" % p)
                    plugins[p].update()
        else:
            if not os.path.isdir(configdir):
                os.makedirs(configdir)
            if not os.path.isfile(configpath):
                origpath = os.path.join(
                    os.path.realpath(__file__)[:-18],
                    'data/example.conf'
                )
                copyfile(origpath, configpath)
            subprocess.call(os.environ.get('EDITOR', 'vi') + ' ' + configpath, shell=True)



#! /usr/bin/env python
import sys
import json
import hashlib
from harpoon.commands.base import Command
from harpoon.lib.utils import json_serial, typeguess, unbracket
from pythreatgrid2 import ThreatGrid, ThreatGridError


class CommandThreatGrid(Command):
    """
    # Threat Grid

    **Search in Threat Grid database through the API**

    * Search for a domain: `harpoon threatgrid search domain DOMAIN`
    * Search for a hash : ``harpoon threatgrid hash HASH`

    """
    name = "threatgrid"
    description = "Request Threat Grid API"
    config = {'ThreatGrid': ['key']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('hash', help='Request info on a hash')
        parser_a.add_argument('HASH', help='Hash')
        parser_a.set_defaults(subcommand='hash')
        parser_b = subparsers.add_parser('search', help='Search in Total Hash database')
        parser_b.add_argument('--json', '-j', action='store_true', help='Show raw json')
        parser_b.add_argument('TYPE', help='type of the query (can be checksum, checksum_sample, path, path_sample, path_artifact, path_deleted, url, registry_key, domain, domain_dns_lookup, domain_http_request, ip, ip_dns_lookup, ip_src, ip_dst, ioc, tag)')
        parser_b.add_argument('QUERY', help='query')
        parser_b.set_defaults(subcommand='search')
        parser_c = subparsers.add_parser('networklist', help='Read list of network indicators (domain or IP) from a file a search for related samples')
        parser_c.add_argument('FILE', help='File')
        parser_c.set_defaults(subcommand='networklist')
        self.parser = parser

    def run(self, conf, args, plugins):
        tg = ThreatGrid(conf['ThreatGrid']['key'])
        if 'subcommand' in args:
            if args.subcommand == "search":
                try:
                    res = tg.search_samples(unbracket(args.QUERY), type=args.TYPE)
                except ThreatGridError:
                    print("Invalid type")
                if args.json:
                    print(json.dumps(res, sort_keys=True, indent=4))
                else:
                    if len(res['items']) == 0:
                        print('Not found')
                    else:
                        already = []
                        for item in res['items']:
                            if item['sample_sha256'] not in already:
                                print("%s - %s - %s" % (item['ts'], item['sample_sha256'], "https://panacea.threatgrid.com/mask/samples/" + item['sample']))
                                already.append(item['sample_sha256'])
            elif args.subcommand == 'hash':
                hash_type = {32: 'md5', 40: 'sha1', 64: 'sha256'}
                res = tg.get_sample(args.HASH, type=hash_type[len(args.HASH)])
                if len(res['items']) > 0:
                    item = res['items'][0]
                    print("Sample submitted the %s: https://panacea.threatgrid.com/mask/samples/%s" % (item['submitted_at'], item['id']))
                    idd = item['id']

                    res = tg.get_sample_threats(idd)
                    print('\nThreats:')
                    for t in res['bis']:
                        print("-%s" % t)
                else:
                    print('Hash not found')
            elif args.subcommand == 'networklist':
                with open(args.FILE, 'r') as f:
                    data = f.read().split('\n')
                for d in data:
                    target = unbracket(d.strip())
                    gtype = typeguess(target)
                    print(target)
                    res = tg.search_samples(target, type=gtype)
                    if len(res['items']) > 0:
                        already = []
                        for item in res['items']:
                            if item['sample_sha256'] not in already:
                                print("-%s: https://panacea.threatgrid.com/mask/samples/%s %s" % (item['ts'][:10], item['sample'], item['sample_sha256']))
                                already.append(item['sample_sha256'])

                    else:
                        print('-Nothing found')
                    print('')
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import os
import sys
import json
from harpoon.commands.base import Command
from harpoon.lib.utils import bracket, unbracket, json_serial
from harpoon.lib.certspotter import CertSpotter, CertSpotterError

class CommandCertSpotter(Command):
    """
    # Cert Spotter Command

    Search in Certificate Transparency database Cert Spotter https://sslmate.com/certspotter. Only actual certificate can be searched without paid plan

    * Search for certificates of a domain : `harpoon certspotter search DOMAIN`
    * Search for certificates of a domain including expired certificates : `harpoon certspotter search DOMAIN -a` (paid plans only)
    * Get information on a certificate: `harpoon certspotter cert SHA256`

    """
    name = "certspotter"
    description = "Get certificates from https://sslmate.com/certspotter"
    config = None

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('search', help='Search certificates for a domain')
        parser_a.add_argument('DOMAIN', help='domain')
        parser_a.add_argument('--all', '-a', help='List all certificates including expired one too (API key needed)', action='store_true')
        parser_a.set_defaults(subcommand='search')
        parser_b = subparsers.add_parser('cert', help='Show information on a certificate')
        parser_b.add_argument('SHA256', help='Sha256 of the certificate')
        parser_b.set_defaults(subcommand='cert')
        self.parser = parser

    def run(self, conf, args, plugins):
        try:
            cs = CertSpotter(conf['CertSpotter']['key'])
        except KeyError:
            cs = CertSpotter()

        if 'subcommand' in args:
            if args.subcommand == 'search':
                if args.all:
                    if cs.authenticated:
                        try:
                            res = cs.list(unbracket(args.DOMAIN), expired=True)
                        except CertSpotterError:
                            print("Error with the API, likely because you need a paid plan to search expired certs. Check censys or crtsh plugins instead")
                            sys.exit(1)
                    else:
                        print("API key needed for expired certificated")
                        sys.exit(1)
                else:
                    res = cs.list(unbracket(args.DOMAIN))
                print(json.dumps(res, sort_keys=True, indent=4, separators=(',', ': '), default=json_serial))
            elif args.subcommand == "cert":
                res = cs.get_cert(args.SHA256)
                print(json.dumps(res, sort_keys=True, indent=4, separators=(',', ': '), default=json_serial))
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import sys
import json
from harpoon.commands.base import Command
from harpoon.lib.googl import GoogleShortener

class CommandGoogl(Command):
    """
    # Goo.gl URL shortener plugin

    * Search for a hash: `harpoon googl -H 12445`
    * Search for a list of hash from a file: `harpoon googl -f FILE`
    """
    name = "googl"
    description = "Requests Google url shortener API"
    config = {'Googl': ['token']}

    def add_arguments(self, parser):
        parser.add_argument('--hash', '-H', help='HASH of a link')
        parser.add_argument('--file', '-f', help='File containing list of hashes')

    def run(self, conf, args, plugins):
        if 'Googl' not in conf:
            print('Invalid configuration file, quitting...')
            sys.exit(1)
        if 'token' not in conf['Googl']:
            print('Invalid configuration file, quitting...')
            sys.exit(1)
        go = GoogleShortener(conf['Googl']['token'])
        if args.hash:
            print(json.dumps(go.get_analytics(args.hash), sort_keys=True, indent=4, separators=(',', ':')))
        elif args.file:
            with open(args.file, 'r') as f:
                data = f.read().split()

            print("Date;Status;Short URL;Long URL;Analytics;Short URL Clicks;Long URL Clicks")
            for d in data:
                res = go.get_analytics(d.strip())
                print("%s;%s;%s;%s;https://goo.gl/#analytics/goo.gl/%s/all_time;%s;%s" %
                    (
                        res.get("created", ""),
                        res.get("status", ""),
                        res["id"],
                        res.get("longUrl", ""),
                        res["id"][-6:],
                        res.get("analytics", {}).get("allTime", {}).get("shortUrlClicks", ""),
                        res.get("analytics", {}).get("allTime", {}).get("longUrlClicks", ""),
                    )
                )
        else:
            print("Please provide a file (-f) or a hash (-H)")



import requests

class UrlScan(object):
    def __init__(self):
        self.url = "https://urlscan.io/api/v1/"

    def search(self, query, size=100, offset=0):
        params = {
            'q': query,
            'size': size,
            'offset': offset
        }
        r = requests.get(self.url + "search/", params=params)
        return r.json()

    def view(self, uid):
        r = requests.get(self.url + 'result/' + uid)
        return r.json()




#! /usr/bin/env python
import sys
import json
import datetime
from harpoon.commands.base import Command
from harpoon.lib.utils import bracket, unbracket
from threatminer import ThreatMiner


class CommandThreatMiner(Command):
    """
    # ThreatMiner

    https://www.threatminer.org/

    * Get report related to an IOC : `harpoon threatminer report DOMAIN`
    * Get samples related to a domain or IP : `harpoon threatminer sample DOMAIN`
    * Get metadata of a file : `harpoon threatminer metadata HASH`
    * Get HTTP traffic for a file : `harpoon threatminer traffic HASH`
    * Get hosts associated with a file : `harpoon threatminer host HASH`
    * Get mutants associated with a file : `harpoon threatminer mutant HASH`
    * Get AV detections associated with a file : `harpoon threatminer av HASH`
    * Get whois info on a domain : `harpoon threatminer whois DOMAIN`
    * Get Passive DNS info on a domain : `harpoon threatminer dns DOMAIN`
    * Get URLs related to a domain : `harpoon threatminer uri DOMAIN`
    * Get subdomains related to a domain : `harpoon threatminer subdomain DOMAIN`
    """
    name = "threatminer"
    description = "Requests TreatMiner database https://www.threatminer.org/"

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('report', help='Request information a Threat Report')
        parser_a.add_argument('INDICATOR', help='Indicator to check')
        parser_a.set_defaults(subcommand='report')
        parser_b = subparsers.add_parser('sample', help='Get samples associated with a domain or IP')
        parser_b.add_argument('INDICATOR', help='Indicator to check')
        parser_b.set_defaults(subcommand='sample')
        parser_c = subparsers.add_parser('metadata', help='Get metadata associated with a file')
        parser_c.add_argument('HASH', help='Hash to get metadata on')
        parser_c.set_defaults(subcommand='metadata')
        parser_d = subparsers.add_parser('traffic', help='Get HTTP traffic associated with a file')
        parser_d.add_argument('HASH', help='Hash to get metadata on')
        parser_d.set_defaults(subcommand='traffic')
        parser_e = subparsers.add_parser('host', help='Get hosts associated with a file')
        parser_e.add_argument('HASH', help='Hash to get info on')
        parser_e.set_defaults(subcommand='host')
        parser_f = subparsers.add_parser('mutant', help='Get mutants associated with a file')
        parser_f.add_argument('HASH', help='Hash to get info on')
        parser_f.set_defaults(subcommand='mutant')
        parser_g = subparsers.add_parser('av', help='Get AV detections associated with a file')
        parser_g.add_argument('HASH', help='Hash to get info on')
        parser_g.set_defaults(subcommand='av')
        parser_h = subparsers.add_parser('whois', help='Get whois info on a domain')
        parser_h.add_argument('DOMAIN', help='Domain')
        parser_h.set_defaults(subcommand='whois')
        parser_i = subparsers.add_parser('dns', help='Get Passive DNS info on a domain')
        parser_i.add_argument('DOMAIN', help='Domain')
        parser_i.set_defaults(subcommand='dns')
        parser_j = subparsers.add_parser('uri', help='Get URLs related to a domain')
        parser_j.add_argument('DOMAIN', help='Domain')
        parser_j.set_defaults(subcommand='uri')
        parser_k = subparsers.add_parser('subdomain', help='Get subdomains related to a domain')
        parser_k.add_argument('DOMAIN', help='Domain')
        parser_k.set_defaults(subcommand='subdomain')
        self.parser = parser


    def run(self, conf, args, plugins):
        if 'subcommand' in args:
            tm = ThreatMiner()
            if args.subcommand == 'report':
                response = tm.get_report(unbracket(args.INDICATOR))
                if response['status_code'] == '200':
                    if len(response['results']) > 0:
                        print("Reports found:")
                        for r in response['results']:
                            print("{} {} - {}".format(
                                r['year'],
                                r['filename'],
                                r['URL']
                            ))
                    else:
                        print("No report found for this indicator")
                elif response['status_code'] == '404':
                    print("No report found for this indicator")
                else:
                    print("Request failed: {}".format(response['status_message']))
            elif args.subcommand == 'sample':
                response = tm.get_related_samples(unbracket(args.INDICATOR))
                if response['status_code'] == '200':
                    if len(response['results']) > 0:
                        print("Samples found:")
                        for r in response['results']:
                            print(r)
                    else:
                        print("No report found for this indicator")
                elif response['status_code'] == '404':
                    print("No report found for this indicator")
                else:
                    print("Request failed: {}".format(response['status_message']))
            elif args.subcommand == 'metadata':
                response = tm.get_metadata(args.HASH)
                if response['status_code'] == '200':
                    for r in response['results']:
                        for d in r:
                            print("{} - {}".format(d, r[d]))
                        print("")
                elif response['status_code'] == '404':
                    print("No report found for this indicator")
                else:
                    print("Request failed: {}".format(response['status_message']))
            elif args.subcommand == 'traffic':
                response = tm.get_http_traffic(args.HASH)
                if response['status_code'] == '200':
                    for t in response['results'][0]['http_traffic']:
                        for d in t.keys():
                            if d != 'raw':
                                print("{} - {}".format(d, t[d]))
                        print("")
                elif response['status_code'] == '404':
                    print("No traffic found for this file")
                else:
                    print("Request failed: {}".format(response['status_message']))
            elif args.subcommand == 'host':
                response = tm.get_hosts(args.HASH)
                if response['status_code'] == '200':
                    print("domains:")
                    for d in response["results"][0]["domains"]:
                        print("{} - {}".format(d["domain"], d["ip"]))
                    print("\nHosts:")
                    for h in response["results"][0]["hosts"]:
                        print(h)
                elif response['status_code'] == '404':
                    print("File not found")
                else:
                    print("Request failed: {}".format(response['status_message']))
            elif args.subcommand == 'mutant':
                response = tm.get_mutants(args.HASH)
                if response['status_code'] == '200':
                    for m in response["results"][0]['mutants']:
                        print(m)
                elif response['status_code'] == '404':
                    print("File not found")
                else:
                    print("Request failed: {}".format(response['status_message']))
            elif args.subcommand == 'av':
                response = tm.get_av_detections(args.HASH)
                if response['status_code'] == '200':
                    for m in response["results"][0]['av_detections']:
                        print(m)
                elif response['status_code'] == '404':
                    print("File not found")
                else:
                    print("Request failed: {}".format(response['status_message']))
            elif args.subcommand == 'whois':
                response = tm.who_is(args.DOMAIN)
                if response['status_code'] == '200':
                    print(json.dumps(response['results'][0]['whois'], indent=4))
                elif response['status_code'] == '404':
                    print("Domain not found")
                else:
                    print("Request failed: {}".format(response['status_message']))
            elif args.subcommand == 'dns':
                response = tm.passive_dns(args.DOMAIN)
                if response['status_code'] == '200':
                    for r in response['results']:
                        print("{} - {} - {}".format(
                            r["ip"],
                            r["first_seen"],
                            r["last_seen"]
                        ))
                elif response['status_code'] == '404':
                    print("Domain not found")
                else:
                    print("Request failed: {}".format(response['status_message']))
            elif args.subcommand == 'uri':
                response = tm.get_uris(args.DOMAIN)
                if response['status_code'] == '200':
                    print(json.dumps(response['results'][0]['whois'], indent=4))
                elif response['status_code'] == '404':
                    print("Domain not found")
                else:
                    print("Request failed: {}".format(response['status_message']))
            elif args.subcommand == 'subdomain':
                response = tm.get_subdomains(args.DOMAIN)
                if response['status_code'] == '200':
                    for s in response['results']:
                        print(s)
                elif response['status_code'] == '404':
                    print("Domain not found")
                else:
                    print("Request failed: {}".format(response['status_message']))
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import sys
import json
from harpoon.commands.base import Command
from pypermacc import Permacc, PermaccError

class CommandPemacc(Command):
    """
    # Perma.cc plugin

    **Query the permacc API**

    * Save an url : harpoon permacc save URL
    * See last 10 urls saved : harpoon permacc list
    * Download an archive for an url: harpoon permacc download Y6JJ-TDUJ
    * Get information on an archive : harpoon permacc info Y6JJ-TDUJ
    """
    name = "permacc"
    description = "Request Perma.cc information through the API"
    config = { 'Permacc': ['key']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('save', help='Save an url in perma.cc')
        parser_a.add_argument('URL', help='Url')
        parser_a.set_defaults(subcommand='save')
        parser_b = subparsers.add_parser('info', help='Show information on a saved page')
        parser_b.add_argument('GUID', help='Guid (like Y6JJ-TDUJ)')
        parser_b.set_defaults(subcommand='info')
        parser_c = subparsers.add_parser('download', help='Download the WARC archive')
        parser_c.add_argument('GUID', help='Guid (like Y6JJ-TDUJ)')
        parser_c.add_argument('--output', '-o', default='webpage.warc', help='Name of the file downloaded')
        parser_c.set_defaults(subcommand='download')
        parser_d = subparsers.add_parser('list', help='List pages archived by your account')
        parser_d.set_defaults(subcommand='list')
        self.parser = parser

    def run(self, conf, args, plugins):
        permacc = Permacc(conf['Permacc']['key'])
        if 'subcommand' in args:
            if args.subcommand == 'save':
                try:
                    saved = permacc.archive_create(args.URL)
                    print('Saved: https://perma.cc/%s - %s' % (saved['guid'], saved['guid']))
                except PermaccError:
                    print('Failed')
            elif args.subcommand == 'download':
                warc = permacc.archive_download(args.GUID)
                if args.output == 'webpage.warc':
                    outputname = args.GUID + '.warc'
                else:
                    outputname = args.output
                with open(outputname, "wb") as f:
                    f.write(warc)
                print('Archive saved as %s' % outputname)
            elif args.subcommand == 'info':
                infos = permacc.archive_detail(args.GUID)
                print(json.dumps(infos, sort_keys=False, indent=4))
            elif args.subcommand == 'list':
                archives = permacc.user_archives()
                print('Last 10 pages:')
                for o in archives['objects']:
                    print("- https://perma.cc/%s %s" % (o['guid'], o['url']))
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='harpoon',
    version='0.1.5',
    description='Another OSINT CLI tool',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/Te-k/harpoon',
    author='Tek',
    author_email='tek@randhome.io',
    keywords='osint',
    include_package_data=True,
    install_requires=[
        'requests',
        'configparser',
        'tweepy',
        'passivetotal',
        'beautifulsoup4==4.9.1',
        'lxml==4.5.1',
        'censys<1.0.0',
        'shodan',
        'fullcontact.py',
        'pyhunter',
        'PyGitHub',
        'telethon==0.18.3',
        'virustotal-api',
        'mispy',
        'OTXv2',
        'IPy',
        'maxminddb>=1.4.0',
        'pyasn',
        'spyonweb==0.1',
        'selenium',
        'geoip2',
        'pygreynoisev1==0.1',
        'dnspython',
        'consolemd==0.5.1',
        'pypermacc==0.1.1',
        'archiveis',
        'pytz',
        'pypdns==1.3',
        'pybinaryedge==0.5',
        'spyonweb==0.1',
        'pythreatgrid2==0.1.1',
        'pycrtsh==0.3.2',
        'pysafebrowsing==0.1.1',
        'dnsdb==0.2.5',
        'pysecuritytrails==0.1.3',
        'phonenumbers==8.12.4',
        'threatminer==1.0'
        ],
    python_requires='>=3.5',
    license='GPLv3',
    packages=['harpoon', 'harpoon.commands', 'harpoon.lib', 'harpoon.data'],
    package_dir={'harpoon.lib': 'harpoon/lib'},
    package_data={'harpoon': ['harpoon/data/*.conf']},
    entry_points= {
        'console_scripts': [ 'harpoon=harpoon.main:main' ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)



#! /usr/bin/env python
import sys
import json
from harpoon.commands.base import Command
from pygreynoisev1 import GreyNoise, GreyNoiseError


class CommandGreyNoise(Command):
    """
    # GreyNoise

    See https://github.com/Grey-Noise-Intelligence/api.greynoise.io

    * List tags: `harpoon greynoise -l`
    * Search for an IP: `harpoon greynoise -i IP`
    * Search for a tag with csv output:  `harpoon greynoise -t CENSYS -f csv`
    """
    name = "greynoise"
    description = "Request Grey Noise API"

    def add_arguments(self, parser):
        parser.add_argument('--list', '-l', help="List tags", action='store_true')
        parser.add_argument('--ip', '-i', help="Query an IP address")
        parser.add_argument('--tag', '-t', help="Query a tag")
        parser.add_argument('--format', '-f', help="Output format", choices=["csv", "json"], default="json")
        self.parser = parser

    def run(self, conf, args, plugins):
        gn = GreyNoise()
        if args.list:
            res = gn.tags()
            if args.format == "json":
                print(json.dumps(res, indent=4, sort_keys=True))
            else:
                for i in res:
                    print(i)
        elif args.ip:
            try:
                res = gn.query_ip(args.ip)
            except GreyNoiseError:
                print("IP not found")
            else:
                if args.format == "json":
                    print(json.dumps(res, indent=4, sort_keys=True))
                else:
                    print("Tag;Category;Confidence;Intention;First Seen;Last Seen;ASN;Datacenter;Link;Org;OS;RDNS;Tor")
                    for r in res:
                        print("%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s" % (
                                r["name"],
                                r["category"],
                                r["confidence"],
                                r["intention"],
                                r["first_seen"],
                                r["last_updated"],
                                r["metadata"]["asn"],
                                r["metadata"]["datacenter"],
                                r["metadata"]["link"],
                                r["metadata"]["os"],
                                r["metadata"]["org"],
                                r["metadata"]["rdns"],
                                r["metadata"]["tor"]
                            )
                        )
        elif args.tag:
            try:
                res = gn.query_tag(args.tag)
            except GreyNoiseError:
                print("TAG does not exist")
            else:
                if args.format == "json":
                    print(json.dumps(res, indent=4, sort_keys=True))
                else:
                    print("IP;Tag;Category;Confidence;Intention;First Seen;Last Seen;ASN;Datacenter;Link;Org;OS;RDNS;Tor")
                    for r in res:
                        print("%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s" % (
                                r["ip"],
                                r["name"],
                                r["category"],
                                r["confidence"],
                                r["intention"],
                                r["first_seen"],
                                r["last_updated"],
                                r["metadata"]["asn"],
                                r["metadata"]["datacenter"],
                                r["metadata"]["link"],
                                r["metadata"]["os"],
                                r["metadata"]["org"],
                                r["metadata"]["rdns"],
                                r["metadata"]["tor"]
                            )
                        )
        else:
            self.parser.print_help()



import requests
import json


class CertSpotterError(Exception):
    def __init__(self, message):
        Exception.__init__(self, message)
        self.message = message


class CertSpotter(object):
    def __init__(self, key=''):
        if key == '':
            self.authenticated = False
        else:
            self.authenticated = True
        self.key = key

    def list(self, domain, expired=False, duplicate=False):
        if expired and not self.authenticated:
            raise CertSpotter("You are not allowed to search fo expired certificate without key")
        if self.authenticated:
            r = requests.get(
                "https://certspotter.com/api/v0/certs",
                params = {'domain': domain, 'expired': expired, 'duplicate': duplicate },
                auth=(self.key, '')
            )
        else:
            r = requests.get(
                "https://certspotter.com/api/v0/certs",
                params = {'domain': domain, 'expired': expired, 'duplicate': duplicate }
            )
        if r.status_code == 200:
            return r.json()
        else:
            raise CertSpotterError("Invalid HTTP status code %i" % r.status_code)

    def get_cert(self, sha256):
        r = requests.get("https://certspotter.com/api/v0/certs/" + sha256)
        if r.status_code == 200:
            return r.json()
        else:
            raise CertSpotterError("Invalid HTTP status code %i" % r.status_code)

    def get_cert_pem(self, sha256):
        r = requests.get("https://certspotter.com/api/v0/certs/" + sha256 + '.pem')
        if r.status_code == 200:
            return r.text
        else:
            raise CertSpotterError("Invalid HTTP status code %i" % r.status_code)

    def get_cert_der(self, sha256):
        r = requests.get("https://certspotter.com/api/v0/certs/" + sha256 + '.der')
        if r.status_code == 200:
            return r.text
        else:
            raise CertSpotterError("Invalid HTTP status code %i" % r.status_code)



import argparse
import json
import requests
import os
import sys
import datetime
import logging
try:
    from urllib.parse import urljoin
except ImportError:
    # python2
    from urlparse import urljoin
try:
    import http.client as http_client
except ImportError:
    # Python 2
    import httplib as http_client
try:
    import configparser as ConfigParser
except ImportError:
    import ConfigParser


class Error(Exception):
    pass


class BitlyError(Error):
    def __init__(self, code, message):
        Error.__init__(self, message)
        self.code = code


class Bitly(object):
    """
    Python class managing bitly API calls
    Mostly inspired from https://github.com/bitly/bitly-api-python
    """

    def __init__(self, access_token):
        self.access_token = access_token
        self.host = 'https://api-ssl.bit.ly/'
        self.linkbase = "http://bit.ly/"

    def _apicall(self, url, params={}):
        params["access_token"] = self.access_token
        r = requests.get(urljoin(self.host, url), params=params)
        data = r.json()
        if data["status_code"] == 200:
            return data['data']
        else:
            raise BitlyError(data["status_code"], data["status_txt"])

    def pprint(self, data):
        print(
            json.dumps(
                data,
                sort_keys=True,
                indent=4,
                separators=(',', ': ')
            )
        )

    def user_info(self, login=None):
        if login is not None:
            return self._apicall("/v3/user/info", params={"login": login})
        else:
            return self._apicall("/v3/user/info")

    def user_link_history(self, user=None):
        if user is not None:
            return self._apicall("/v3/user/link_history", params={"user": user})
        else:
            return self._apicall("/v3/user/link_history")

    def info(self, hash):
        return self._apicall("/v3/info", {"shortUrl": self.linkbase + hash, "expand_user": True})

    def link_expand(self, hash):
        return self._apicall("/v3/expand", {"shortUrl": self.linkbase + hash})

    def link_info(self, hash):
        return self._apicall("/v3/link/info", {"link": self.linkbase + hash})

    def link_clicks(self, hash):
        return self._apicall("/v3/link/clicks", {"link": self.linkbase + hash})

    def link_countries(self, hash):
        return self._apicall("/v3/link/countries", {"link": self.linkbase + hash})

    def link_referrers(self, hash):
        return self._apicall("/v3/link/referrers", {"link": self.linkbase + hash})

    def link_encoders(self, hash):
        return self._apicall("/v3/link/encoders", {"link": self.linkbase + hash, "expand_user": True})

    def link_encoders_by_count(self, hash):
        return self._apicall("/v3/link/encoders_by_count", {"link": self.linkbase + hash})

    def link_encoders_count(self, hash):
        return self._apicall("/v3/link/encoders_count", {"link": self.linkbase + hash})

    def link_lookup(self, long_url):
        return self._apicall("/v3/user/link_lookup", {"url": long_url})


class Link(object):
    def __init__(self, api, hash):
        self._hash = hash.strip()
        self._api = api
        self._clicks = None
        self._is_aggregate = None
        self._aggregate = None
        self._user_hash = None
        self._is_user_valid = None
        self._user_info = None
        self._title = None
        self._timestamp = None
        self._referrers = None
        self._countries = None
        self._encoders_count = None
        self._long_url = None
        self._infos = None

    @property
    def short_url(self):
        return ("http://bit.ly/%s" % self.hash)

    @property
    def long_url(self):
        if self._long_url is None:
            res = self._api.link_expand(self.hash)
            if len(res["expand"]) > 1:
                print("Too much data here, weird...")
            data = res["expand"][0]

            self._long_url = data["long_url"]
            if self._is_aggregate is None:
                if data["global_hash"] == self.hash:
                    self._is_aggregate = True
                else:
                    self._is_aggregate = False
                    self._aggregate = Link(self._api, data["global_hash"])
                if self._user_hash is None:
                    self._user_hash = data["user_hash"]
        return self._long_url

    @property
    def is_user_valid(self):
        if self._is_user_valid is None:
            try:
                data = self._api.user_info(self.user_hash)
                self._is_user_valid = True
                self._user_info = data
            except BitlyError:
                self._is_user_valid = False

    @property
    def infos(self):
        if self._infos is None:
            self._infos = self._api.link_info(self.hash)
        return self._infos

    @property
    def user_info(self):
        if self._user_info is None:
            self.is_user_valid
        return self._user_info

    @property
    def hash(self):
        return self._hash

    @property
    def clicks(self):
        if self._clicks is None:
            data = self._api.link_clicks(self.hash)
            self._clicks = data["link_clicks"]
        return self._clicks

    @property
    def is_aggregate(self):
        if self._is_aggregate is None:
            self._get_infos()
        return self._is_aggregate

    @property
    def aggregate(self):
        if self._is_aggregate is None:
            self._get_infos()
        return self._aggregate

    @property
    def title(self):
        if self._is_aggregate is None:
            self._get_infos()
        return self.title

    @property
    def timestamp(self):
        if self._timestamp is None:
            self._get_infos()
        return self._timestamp

    @property
    def referrers(self):
        if self._referrers is None:
            data = self._api.link_referrers(self.hash)
            self._referrers = data["referrers"]
        return self._referrers

    @property
    def countries(self):
        if self._countries is None:
            data = self._api.link_countries(self.hash)
            self._countries = data["countries"]
        return self._countries

    @property
    def encoders_count(self):
        if self._encoders_count is None:
            data = self._api.link_encoders_count(self.hash)
            self._encoders_count = data["count"]
            if self._aggregate is None:
                self._aggregate = Link(self._api, data["aggregate_link"][-6:])
        return self._encoders_count

    @property
    def user_hash(self):
        if self._user_hash is None:
            self._get_infos()
        return self._user_hash

    def _get_infos(self):
        res = self._api.info(self.hash)
        data = res["info"][0]
        if len(res["info"]) > 1:
            # FIXME : logger here
            print("More than one info for this link, weird")
        self._timestamp = datetime.datetime.fromtimestamp(data["created_at"])
        if data["global_hash"] == self.hash:
            self._is_aggregate = True
        else:
            self._is_aggregate = False
            self._aggregate = Link(self._api, data["global_hash"])
        self._title = data["title"]
        self._user_hash = data["user_hash"]

    def __repr__(self):
        return "%s(%s)" % ("Link", self.short_url)

    def pprint(self):
        print("-------------------- Bit.ly Link infos -------------------")
        print("# INFO")
        print("Link: %s\t\tMetrics: %s+" % (self.short_url, self.short_url))
        print("Expanded url: %s" % self.long_url)
        print("Creation Date: %s" % str(self.timestamp))
        if self.is_aggregate:
            print("Aggregate link")
        else:
            print("Aggregate link: %s" % self.aggregate.short_url)
        print("%i bitly redirect to this url" % self.encoders_count)
        print("\n# LINK  INFO")
        for i in self.infos:
            print("%s: %s" % (i, self.infos[i]))

        print("\n# USERS")
        print("User: %s" % self.user_hash)
        if self.is_user_valid:
            print("Valid user:")
            for p in self.user_info:
                print("\t%s : %s" % (p, self.user_info[p]))
        else:
            print("Invalid user!")
        print("\n# CLICKS")
        print("%i clicks on this link" % self.clicks)
        print("\n# COUNTRIES")
        for c in self.countries:
            print("-%s: %i clicks" % (c["country"], c["clicks"]))
        print("\n# REFERRERS")
        for r in self.referrers:
            if "referrer" in r:
                print("-%s: %i clicks" % (r["referrer"], r["clicks"]))
            else:
                print("-%s (%s): %i clicks" % (r["referrer_app"], r["url"], r["clicks"]))



#! /usr/bin/env python
import sys
import json
import hashlib
from harpoon.commands.base import Command
from harpoon.lib.hybrid import HybridAnalysis, HybridAnalysisFailed
from harpoon.lib.utils import json_serial


class CommandHybridAnalysis(Command):
    """
    # HybridAnalysis

    **Query Hybrid Analysis Sandbox information (https://www.hybrid-analysis.com/)**

    Requests:
    * `harpoon hybrid hash 52c408a3dd38743a179ef800b390bbac` : Give information on a hash
    * `harpoon hybrid quota` : show quota information
    * `harpoon hybrid search host:95.181.53.78` : search in Hybrid Analysis samples
    * `harpoon analysis 52c408a3dd38743a179ef800b390bbac 100` : gived detailed sandbox report

    Search operators :
    * host:95.181.53.78
    * port:3448
    * domain:checkip.dyndns.org
    * vxfamily:upatre
    * indicatorid:network-6 (Show all reports matching 'Contacts Random Domain Names')
    * filetype:jar
    * filetype_tag:hwp
    * url:google
    * similar-to:59f629287c1ce2bcb39e4bc41633756e516bf04909750eba1cd2c787d02d3347
    * authentihash:hash
    * tag:teslacrypt
    """
    name = "hybrid"
    description = "Requests Hybrid Analysis platform"
    config = {'HybridAnalysis': ['key', 'secret']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('hash', help='Request info on a hash')
        parser_a.add_argument('HASH', help='Hash')
        parser_a.set_defaults(subcommand='hash')
        parser_b = subparsers.add_parser('analysis', help='Request details on an analysis')
        parser_b.add_argument('HASH', help='Hash')
        parser_b.set_defaults(subcommand='analysis')
        parser_c = subparsers.add_parser('help', help='Help on search terms')
        parser_c.set_defaults(subcommand='help')
        parser_f = subparsers.add_parser('quota', help='Print information on Hybrid Analysis Quota')
        parser_f.set_defaults(subcommand='quota')
        parser_e = subparsers.add_parser('search', help='Search in Total Hash database')
        parser_e.add_argument('QUERY', help='query')
        parser_e.set_defaults(subcommand='search')
        self.parser = parser

    def run(self, conf, args, plugins):
        ha = HybridAnalysis(conf['HybridAnalysis']['key'], conf['HybridAnalysis']['secret'])
        if 'subcommand' in args:
            if args.subcommand == 'quota':
                res = ha.quota()
                print(json.dumps(res, sort_keys=True, indent=4, separators=(',', ': '), default=json_serial))
            elif args.subcommand == "search":
                try:
                    res = ha.search(args.QUERY)
                except HybridAnalysisFailed:
                    print("File not found")
                else:
                    print(json.dumps(res, sort_keys=True, indent=4, separators=(',', ': '), default=json_serial))
            elif args.subcommand == 'hash':
                try:
                    res = ha.get_report(args.HASH)
                except HybridAnalysisFailed:
                    print("File not found")
                else:
                    print(json.dumps(res, sort_keys=True, indent=4, separators=(',', ': '), default=json_serial))
            elif args.subcommand == 'analysis':
                try:
                    res = ha.get_last_analysis(args.HASH)
                except HybridAnalysisFailed:
                    print("File not found")
                else:
                    print(json.dumps(res, sort_keys=True, indent=4, separators=(',', ': '), default=json_serial))
            elif args.subcommand == 'help':
                print("""
Example of search operators:
host:95.181.53.78
port:3448
domain:checkip.dyndns.org
vxfamily:upatre
indicatorid:network-6 (Show all reports matching 'Contacts Random Domain Names')
filetype:jar
filetype_tag:hwp
url:google
similar-to:59f629287c1ce2bcb39e4bc41633756e516bf04909750eba1cd2c787d02d3347
authentihash:hash
tag:teslacrypt
                """)
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import sys
import json
import hashlib
from harpoon.commands.base import Command
from harpoon.lib.malshare import MalShare, MalShareFailed, MalShareNotFound, MalShareSampleMissing
from harpoon.lib.utils import json_serial


class CommandMalShare(Command):
    """
    # MalShare

    **Requests information from MalShare (https://malshare.com/)**

    * `harpoon malshare search beget.tech` : Search in the database
    * `harpoon malshare hash 924c1fb188fb8dcbcee616308389fc22` : Information about a hash
    * `harpoon  malshare download dde72ae232dc63298465861482d7bb93 -o infected` : ;Download the sample
    """
    name = "malshare"
    description = "Requests MalShare database"
    config = {'MalShare': ['key']}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help='Subcommand')
        parser_a = subparsers.add_parser('hash', help='Request info on a hash')
        parser_a.add_argument('HASH', help='Hash')
        parser_a.set_defaults(subcommand='hash')
        parser_b = subparsers.add_parser('download', help='Download the given hash')
        parser_b.add_argument('HASH', help='Hash')
        parser_b.add_argument('--output', '-o', help='Output file name')
        parser_b.set_defaults(subcommand='download')
        parser_e = subparsers.add_parser('search', help='Search in Total Hash database')
        parser_e.add_argument('QUERY', help='query')
        parser_e.set_defaults(subcommand='search')
        self.parser = parser

    def run(self, conf, args, plugins):
        ms = MalShare(conf['MalShare']['key'])
        if 'subcommand' in args:
            if args.subcommand == "search":
                try:
                    res = ms.search(args.QUERY)
                except MalShareFailed:
                    print("File not found")
                else:
                    print(json.dumps(res, sort_keys=True, indent=4, separators=(',', ': '), default=json_serial))
            elif args.subcommand == 'hash':
                try:
                    res = ms.file_info(args.HASH)
                except MalShareNotFound:
                    print("File not found")
                else:
                    print(json.dumps(res, sort_keys=True, indent=4, separators=(',', ': '), default=json_serial))
            elif args.subcommand == 'download':
                try:
                    data = ms.download(args.HASH)
                except MalShareNotFound:
                    print("File not found")
                except MalShareSampleMissing:
                    print("Missing sample, it is a bug")
                else:
                    outfile = args.output if args.output else args.HASH
                    with open(outfile, "w") as f:
                        f.write(data)
                    print("Sample created %s" % outfile)
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()
